/*
 * eth_ecat_pdo_fmmu.hpp
 *
 *  Created on: May 16, 2023
 *      Author: algin
 */

#ifndef FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_PDO_FMMU_HPP_
#define FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_PDO_FMMU_HPP_

#include "ethernet_industry/eth_ecat_buffer.hpp"

namespace free_rtos {

namespace ecat_pdo_fmmu {

class EcatPdoFMMUSlave {
public:
    EcatPdoFMMUSlave(ecat_buffer::EcatBufferSlave& buffer_slave)
    : buffer_slave_(buffer_slave) { }

    template<typename... DataTypes>
    void write(datagram::EcatTelegram& telegram, DataTypes&... data) {
        ecat_buffer::FMMUProperties& fmmu_properties = buffer_slave_.get_fmmu_properties_write();
        address::Logical logical = fmmu_properties.address;

        datagram::EcatDatagram<command::LWR, DataTypes...> datagram{ {{logical}}, data... };

        do {
            telegram.transfer(datagram);
        } while(datagram.get_wkc() < 0x0001);

        DebugP_log("datagram.get_wkc() = %d\r\n", datagram.get_wkc());
    }

    template<typename... DataTypes>
    void read(datagram::EcatTelegram& telegram, DataTypes&... data) {
        ecat_buffer::FMMUProperties& fmmu_properties = buffer_slave_.get_fmmu_properties_read();
        address::Logical logical = fmmu_properties.address;

        datagram::EcatDatagram<command::LRD, DataTypes...> datagram{ {{logical}}, data... };

        do {
            telegram.transfer(datagram);
        } while(datagram.get_wkc() < 0x0001);
    }

private:
    ecat_buffer::EcatBufferSlave& buffer_slave_;
};

class EthEcatPdoFMMU {
public:
    EthEcatPdoFMMU(ecat_buffer::EthEcatBuffer& ecat_mailbox): ecat_buffer_{ecat_mailbox} { }

    void init();

    template<typename... DataTypes>
    void write(DataTypes&... data) {
        datagram::EcatTelegram& telegram = ecat_buffer_.get_ecat().get_telegram();

        pdo_fmmu_slaves_[0].write(telegram, data...);
    }

    template<typename... DataTypes>
    void read(DataTypes&... data) {
        datagram::EcatTelegram& telegram = ecat_buffer_.get_ecat().get_telegram();

        pdo_fmmu_slaves_[0].read(telegram, data...);
    }

private:
    ecat_buffer::EthEcatBuffer& ecat_buffer_;

    std::vector<EcatPdoFMMUSlave> pdo_fmmu_slaves_;
};

} // namespace ecat_pdo_fmmu

} // namespace free_rtos

#endif /* FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_PDO_FMMU_HPP_ */
